// File: main_cona.cpp
#include <cstdlib>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <time.h>
#include <string.h>
#include <sstream>
#include "graph.h"
using namespace std;


char *nfile     = NULL;
char *pfile     = NULL;
char *n2cfile   = NULL;
char *c2nfile   = NULL;
double p        = 0.55;

void
usage(char *more) {
	cerr << more;
	cerr << "-inf input network file ."<< endl;
	cerr << "-ipf input partition file ."<< endl;
	cerr << "-on2cf output n2c community file ."<< endl;
	cerr << "-oc2nf output c2n community file ."<< endl;
	cerr << "-p threshold parameter ."<<endl;
	exit(0);
}


void
parse_args(int argc, char **argv) {
  if (argc<2)
    usage("Bad arguments number\n");
  string temp;

  for (int i = 1; i < argc; i++) {
    if(argv[i][0] == '-') {
		temp=argv[i];
		if(temp=="-inf"){
			if (i==argc-1)
	           usage("Input network file missing\n");
	        nfile = argv[i+1];
	        i++;
		}else if(temp=="-ipf"){
			if(i==argc-1)
				usage("Input partition file missing\n");
			pfile = argv[i+1];
			i++;
		}else if(temp=="-on2cf"){
			if (i==argc-1)
	           usage("Output n2c file missing\n");
            n2cfile = argv[i+1];
	        i++;
		}else if(temp=="-oc2nf"){
			if (i==argc-1)
	           usage("Output c2n file missing\n");
            c2nfile = argv[i+1];
	        i++;
		}else if(temp=="-p"){
			if (i==argc-1)
	           usage( "threshold parameter missing\n");
            p =atof(argv[i+1]);
	        i++;
		}else{
		  usage("Unknown option\n");
		}
    } else {
        usage("lack command label \"-?\" \n");
    }
  }
}

void
show_time(const char *str) {
	time_t rawtime;
	time ( &rawtime );
	cerr << str << " : " << ctime (&rawtime);
}

double
minVal(double a, double b){
	if(a>=b)
		return b;
	else
		return a;
}

double maxVal(double a, double b){
	if(a>=b)
		return a;
	else
		return b;
}


void
outputCommunities(vector<vector<unsigned int> > &c2n,vector<vector<unsigned int> > &finaln2c,Graph &g,double p,char* n2cfile, char* c2nfile);

void
outputN2c(vector<vector<unsigned int> > &finaln2c,Graph &g,double p,char* n2cfile);

void
outputC2n(vector<vector<unsigned int> > &c2n,vector<vector<unsigned int> > &finaln2c, Graph &g,double p, char* c2nfile);


vector<vector<unsigned int> > 
findOverlappingNodes(vector<vector<unsigned int> > &c2n,vector<unsigned int> &n2c, Graph &g,vector<int> &insideDegree,double p,vector<map<unsigned int,unsigned int> > &internode2outComm,vector<set<unsigned int> > &communityGraph);


int main(int argc, char *argv[])
{

	
	parse_args(argc,argv);
	time_t st,et;
    st=clock();
    srand(time(NULL));
	//vector<vector<unsigned int> > finaln2c;

	show_time("CONA algorithm start");

	
	//1. read network file 

	Graph g(nfile);

	g.nb_links=0;
	g.nb_nodes=0;

	for(unsigned int i=0 ; i<g.links.size(); i++){
		g.nb_links+=g.links[i].size();
		if(g.links[i].size()!=0)
			g.nb_nodes++;
	}

	g.nb_links=g.nb_links/2;

	cout<<"number of nodes = "<<g.nb_nodes<<endl;
	cout<<"number of links = "<<g.nb_links<<endl;

	show_time("Reading node graph completed");




	
	//2. read disjoint community file

	vector<unsigned int> n2c;
	vector<vector<unsigned int> > c2n;

	n2c.resize(g.nb_nodes);

	ifstream finput;
	
	finput.open(pfile,fstream::in);
	if(!finput){
		cerr<<"open pfile failed!"<<endl;
		exit(0);
	}
	
	string node,community;
	int maxcommunity=0;
	string line;

	while(getline(finput,line)){

	  int pos=-1;
	  pos=line.find_first_of(' ');

	  if(pos==-1){
		cerr<<"data error, lacing separator"<<endl;
		exit(0);
	  }

	  node=line.substr(0,pos);
	  community=line.substr(pos+1,line.size()-pos-1);

	  n2c[g.nodeMap[node]]=atoi(community.c_str());  

	  if(atoi(community.c_str()) > maxcommunity)
		  maxcommunity=atoi(community.c_str());

	}   

	c2n.resize(maxcommunity+1);

	for(unsigned int i=0; i<n2c.size(); i++)
		c2n[n2c[i]].push_back(i);

	for(unsigned int i=0; i<c2n.size(); i++)
		sort(c2n[i].begin(),c2n[i].end());
	
	show_time("reading communities complete!");





	//3. compute in-community degree, out-community degree and community graph
	
	st=clock();

	vector<int> insideDegree;
	insideDegree.resize(g.nb_nodes);

	vector<map<unsigned int,unsigned int> > internode2outComm;
	internode2outComm.resize(g.nb_nodes);

	vector<set<unsigned int> > communityGraph;
	communityGraph.resize(c2n.size());


	for(unsigned int i=0; i<g.nb_nodes; i++){

		map<unsigned int,unsigned int> outCommunities;
		int count=0;

		for(unsigned int j=0;j<g.links[i].size(); j++){
			if(n2c[i]!=n2c[g.links[i][j]]){

				//construct community Graph
				if(n2c[i]>n2c[g.links[i][j]])
					communityGraph[n2c[g.links[i][j]]].insert(n2c[i]);
				else if(n2c[i]<n2c[g.links[i][j]])
					communityGraph[n2c[i]].insert(n2c[g.links[i][j]]);

				map<unsigned int,unsigned int>::iterator it;
				it=outCommunities.find(n2c[g.links[i][j]]);

				if(it==outCommunities.end()){
					outCommunities.insert(make_pair(n2c[g.links[i][j]],1));
				}else{
					outCommunities[n2c[g.links[i][j]]]+=1;
				}

			}else{
				count++;
			}

		}

		internode2outComm[i]=outCommunities;
		insideDegree[i]=count;
		
	}
	show_time("computing in-community degree, out-community degree and community graph complete!");

	
	vector<vector<unsigned int> > finaln2c=findOverlappingNodes(c2n,n2c,g,insideDegree,p,internode2outComm,communityGraph);

	et=clock();
	cout<<"time of finding overlapping nodes :"<<(double)difftime(et,st)/(double)1000<<" second"<<endl;
	
	show_time("finding overlapping node complete!");

	//output overlapping communities
	if(n2cfile!=NULL && c2nfile!=NULL){

		outputCommunities(c2n,finaln2c, g,p,n2cfile,c2nfile);
	}
	else if(n2cfile!=NULL && c2nfile==NULL){

		outputN2c(finaln2c,g,p,n2cfile);
	}
	else if(n2cfile==NULL && c2nfile!=NULL){

		outputC2n(c2n,finaln2c,g,p,c2nfile);
	}

	cout<<"output communities complete!"<<endl;
	
	//system("PAUSE");
	return EXIT_SUCCESS;
}



void
outputCommunities(vector<vector<unsigned int> > &c2n,vector<vector<unsigned int> > &finaln2c,Graph &g,double p,char* n2cf, char* c2nf){
    

	ofstream foutn2c;
	foutn2c.open(n2cf,fstream::out);

	int overnodesCount=0;

	for(unsigned int i=0; i<finaln2c.size(); i++){

		if(finaln2c[i].size()>1)
			overnodesCount++;

		foutn2c<<g.converseNodeMap[i];

		for(unsigned int j=0; j<finaln2c[i].size(); j++){
			foutn2c<<" "<<finaln2c[i][j];
		}
		foutn2c<<endl;

		
	}
	foutn2c.close();

	cout<<"number of found overlapping nodes :"<<overnodesCount<<endl;

	ofstream foutc2n;
	foutc2n.open(c2nf,fstream::out);
	
	for(unsigned int i=0; i<c2n.size(); i++){
		
		if(c2n[i].size()>=1){
			foutc2n<<g.converseNodeMap[c2n[i][0]];
		}

		for(unsigned int j=1; j<c2n[i].size(); j++){
			foutc2n<<" "<<g.converseNodeMap[c2n[i][j]];
			
		}

		foutc2n<<endl;
	}

	foutc2n.close();

}



void
outputN2c(vector<vector<unsigned int> > &finaln2c, Graph &g,double p,char* n2cf){


	ofstream fout;
	fout.open(n2cf,fstream::out);

	int overnodesCount=0;

	for(unsigned int i=0; i<finaln2c.size(); i++){

		if(finaln2c[i].size()>1)
			overnodesCount++;

		fout<<g.converseNodeMap[i];

		for(unsigned int j=0; j<finaln2c[i].size(); j++){
			fout<<" "<<finaln2c[i][j];
		}
		fout<<endl;
	}
	fout.close();
	
	cout<<"finding overlapping nodes :"<<overnodesCount<<endl;
	
}





void
outputC2n(vector<vector<unsigned int> > &c2n,vector<vector<unsigned int> > &finaln2c,Graph &g,double p, char* c2nf){

	
	ofstream foutputc2n;
	foutputc2n.open(c2nf,fstream::out);

	for(unsigned int i=0; i<c2n.size(); i++){

		if(c2n[i].size()>=1)
			foutputc2n<<g.converseNodeMap[c2n[i][0]];

		for(unsigned int j=1; j<c2n[i].size(); j++)
			foutputc2n<<" "<<g.converseNodeMap[c2n[i][j]];

		foutputc2n<<endl;
	}

	foutputc2n.close();

	int overnodesCount=0;
	for(unsigned int i=0; i<finaln2c.size(); i++)
		if(finaln2c[i].size()>1)
			overnodesCount++;

	cout<<"finding overlapping nodes :"<<overnodesCount<<endl;
	
}



vector<vector<unsigned int> > 
findOverlappingNodes(vector<vector<unsigned int> > &c2n,vector<unsigned int> &n2c, Graph &g,vector<int> &insideDegree,double p,vector<map<unsigned int,unsigned int> > &internode2outComm,vector<set<unsigned int> > &communityGraph){
    

	
	map<unsigned int, set<unsigned int> > overlappingNodes;
	double percent=p;


	for(unsigned int i=0; i<communityGraph.size();i++){

		for(set<unsigned int>::iterator it=communityGraph[i].begin(); it!=communityGraph[i].end(); it++){


			int di=0,dj=0;
			vector<unsigned int> interI,interJ;


			map<unsigned int,pair<double,double> > interNodes;//pair<out,in>
			set<unsigned int> tempInter;

			for(unsigned int k=0; k<c2n[i].size(); k++){

				map<unsigned int,unsigned int>::iterator it1;
				it1=internode2outComm[c2n[i][k]].find(*it);

				if(it1!=internode2outComm[c2n[i][k]].end()){

					interI.push_back(c2n[i][k]);

					interNodes.insert(make_pair(c2n[i][k],make_pair(it1->second,(double)insideDegree[c2n[i][k]])));

					tempInter.insert(c2n[i][k]);
					
				}
			}
			for(unsigned int k=0;k<c2n[*it].size(); k++){

				map<unsigned int,unsigned int>::iterator it1;
				it1=internode2outComm[c2n[*it][k]].find(i);

				if(it1!=internode2outComm[c2n[*it][k]].end()){

					interJ.push_back(c2n[*it][k]);

					interNodes.insert(make_pair(c2n[*it][k],make_pair(it1->second,(double)insideDegree[c2n[*it][k]])));

					tempInter.insert(c2n[*it][k]);
				}
			}
			map<unsigned int,pair<int,int> > out_in_numoverlinks;//pair<out,in>


			set<unsigned int> overBounN;
			set<unsigned int> overInnerN;
				

			bool LoopOverBoun=true;
			//first, identify boundary overlapping nodes

			while(LoopOverBoun){

				LoopOverBoun=false;
				//delete procedure
				bool loop1=true;

				while(loop1){

					loop1=false;
					set<pair<double,unsigned int> > OverPij;
					for(set<unsigned int>::iterator itt1=overBounN.begin(); itt1!=overBounN.end(); itt1++){
						

						if((interNodes[*itt1].second-out_in_numoverlinks[*itt1].second)==0){
							continue;
						}
						OverPij.insert( make_pair((double)(interNodes[*itt1].first-out_in_numoverlinks[*itt1].first)/(double)(interNodes[*itt1].second-out_in_numoverlinks[*itt1].second),*itt1) );
					}


					if(OverPij.size()!=0 && (*OverPij.begin()).first < percent){

						
						loop1=true;
						LoopOverBoun=true;
						


						for(int k=0; k<g.links[(*OverPij.begin()).second].size(); k++){
							
							if(n2c[(*OverPij.begin()).second]==n2c[g.links[(*OverPij.begin()).second][k]]){
								out_in_numoverlinks[g.links[(*OverPij.begin()).second][k]].second--;
							}else
								out_in_numoverlinks[g.links[(*OverPij.begin()).second][k]].first--;
						}
						
						//add to tempInter
						tempInter.insert((*OverPij.begin()).second);
						//delete from overBounN
						overBounN.erase((*OverPij.begin()).second);

						

					}
				}


				//add procedure
				set<unsigned int> candidate;
				for(set<unsigned int>::iterator itt1=tempInter.begin(); itt1!=tempInter.end(); itt1++){

					if((double)(interNodes[*itt1].first-out_in_numoverlinks[*itt1].first)/(double)(interNodes[*itt1].second-out_in_numoverlinks[*itt1].second) >= percent)
						candidate.insert(*itt1);
				}

				bool loop2=true;
				while(loop2){
					loop2=false;


					double maxp=-1;
					unsigned int maxNode;
					for(set<unsigned int>::iterator itt1=candidate.begin(); itt1!=candidate.end(); itt1++){

						if((interNodes[*itt1].second-(double)out_in_numoverlinks[*itt1].second)==0){
							maxNode=*itt1;
							maxp=percent;
							break;
						}

						if((double)(interNodes[*itt1].first-out_in_numoverlinks[*itt1].first)/(double)(interNodes[*itt1].second-out_in_numoverlinks[*itt1].second) >= maxp){
							maxp=(double)(interNodes[*itt1].first-out_in_numoverlinks[*itt1].first)/(double)(interNodes[*itt1].second-out_in_numoverlinks[*itt1].second);
							maxNode=*itt1;
						}
					}

					if(maxp >= percent){

						loop2=true;
						LoopOverBoun=true;



						//delete from candidate
						candidate.erase(maxNode);
						tempInter.erase(maxNode);

						//add to overBounN
						overBounN.insert(maxNode);
						

						//update out_in_numoverlinks, di,dj, candidate
						{
							
							if(n2c[maxNode]==i){                       //if maxNode belongs to community i

								vector<unsigned int> temp;
								set_intersection(g.links[maxNode].begin(),g.links[maxNode].end(),c2n[*it].begin(),c2n[*it].end(),back_inserter(temp));


								for(unsigned int k=0; k<temp.size(); k++){
									
									out_in_numoverlinks[temp[k]].first++;
																	
								}
								vector<unsigned int> temp2;
								set_intersection(g.links[maxNode].begin(),g.links[maxNode].end(),c2n[i].begin(),c2n[i].end(), back_inserter(temp2));

								for(unsigned int k=0; k<temp2.size(); k++){

									set<unsigned int>::iterator itt2 = overBounN.find(temp2[k]);
									if(itt2==overBounN.end())
										candidate.insert(temp2[k]);

									out_in_numoverlinks[temp2[k]].second++;
									
								}
							}else{                                     //if maxNode belongs to community j

								vector<unsigned int> temp;
								set_intersection(g.links[maxNode].begin(),g.links[maxNode].end(),c2n[i].begin(),c2n[i].end(),back_inserter(temp));


								for(unsigned int k=0; k<temp.size(); k++){

									out_in_numoverlinks[temp[k]].first++;
									
								}
								vector<unsigned int> temp2;
								set_intersection(g.links[maxNode].begin(),g.links[maxNode].end(),c2n[*it].begin(),c2n[*it].end(), back_inserter(temp2));


								for(unsigned int k=0; k<temp2.size(); k++){

									set<unsigned int>::iterator itt2 = overBounN.find(temp2[k]);
									if(itt2==overBounN.end())
										candidate.insert(temp2[k]);
									out_in_numoverlinks[temp2[k]].second++;
								}
							
							}
						}

					}
				}
			}
			
			//identify inner overlapping nodes

			vector<unsigned int> innerI, innerJ;

			set_difference(c2n[i].begin(),c2n[i].end(),interI.begin(),interI.end(),back_inserter(innerI));
			set_difference(c2n[*it].begin(),c2n[*it].end(),interJ.begin(),interJ.end(),back_inserter(innerJ));

			set<unsigned int> innerOverCand;
			
			for(unsigned int k=0; k<innerI.size(); k++)
				innerOverCand.insert(innerI[k]);
			for(unsigned int k=0; k<innerJ.size(); k++)
				innerOverCand.insert(innerJ[k]);

			for(set<unsigned int>::iterator itt6=innerOverCand.begin(); itt6!=innerOverCand.end(); itt6++){
				if(g.links[*itt6].size()==out_in_numoverlinks[*itt6].second){
					overInnerN.insert(*itt6);
				}
			}
			

			for(set<unsigned int>::iterator itt5=overBounN.begin(); itt5!=overBounN.end(); itt5++){
				map<unsigned int, set<unsigned int> >::iterator it2;
				it2=overlappingNodes.find(*itt5);
				if(it2==overlappingNodes.end()){
					set<unsigned int> temp;
					temp.insert(i);
					temp.insert(*it);
					overlappingNodes.insert(make_pair(*itt5,temp));
				}else{
					it2->second.insert(i);
					it2->second.insert(*it);
				}	
			}

			for(set<unsigned int>::iterator itt5=overInnerN.begin(); itt5!=overInnerN.end(); itt5++){
				map<unsigned int, set<unsigned int> >::iterator it2;
				it2=overlappingNodes.find(*itt5);
				if(it2==overlappingNodes.end()){
					set<unsigned int> temp;
					temp.insert(i);
					temp.insert(*it);
					overlappingNodes.insert(make_pair(*itt5,temp));
				}else{
					it2->second.insert(i);
					it2->second.insert(*it);
				}	
			}

		}//j
	}//i

	
	vector<vector<unsigned int> > finaln2c(g.links.size());

	for(unsigned int i=0; i<n2c.size(); i++){
		if(overlappingNodes[i].size()!=0){

			for(set<unsigned int>::iterator it=overlappingNodes[i].begin(); it!=overlappingNodes[i].end(); it++){
				if(*it!=n2c[i]){
					finaln2c[i].push_back(*it);
					c2n[*it].push_back(i);
				}else
					finaln2c[i].push_back(*it);
			}
			
		}else
			finaln2c[i].push_back(n2c[i]);
	}


	//delete totally contained communities
	map<unsigned int, vector<unsigned int> > containedcomm;
	for(unsigned int i=0; i<finaln2c.size(); i++){

		sort(finaln2c[i].begin(),finaln2c[i].end());

		for(unsigned int j=0; j<finaln2c[i].size(); j++){
			map<unsigned int,vector<unsigned int> >::iterator it;
			it=containedcomm.find(finaln2c[i][j]);
			if(it!=containedcomm.end())
			{
				vector<unsigned int> temp;
				set_intersection(finaln2c[i].begin(),finaln2c[i].end(),containedcomm[finaln2c[i][j]].begin(),containedcomm[finaln2c[i][j]].end(),back_inserter(temp));
				containedcomm[finaln2c[i][j]]=temp;
			}else
				containedcomm.insert(make_pair(finaln2c[i][j],finaln2c[i]));
		}
		
	}

	vector<vector<unsigned int> > tempc2n;

	
	for(map<unsigned int,vector<unsigned int> >::iterator it=containedcomm.begin(); it!=containedcomm.end(); it++){
		if(it->second.size()==1)
			tempc2n.push_back(c2n[it->first]);
	}

	c2n.clear();
	c2n=tempc2n;


	finaln2c.clear();
	finaln2c.resize(g.links.size());

	for(unsigned int i=0; i<c2n.size(); i++){
		sort(c2n[i].begin(),c2n[i].end());
		for(unsigned int j=0; j<c2n[i].size(); j++){
			finaln2c[c2n[i][j]].push_back(i);
		}
	}

	for(unsigned int i=0; i<finaln2c.size(); i++)
		sort(finaln2c[i].begin(),finaln2c[i].end());

	return finaln2c;

	
}
